<?php
/**
* Description: Demonstrates building a calendar for a month using the Week class
* Uses UnixTs engine
*/
function getmicrotime(){
    list($usec, $sec) = explode(" ",microtime());
    return ((float)$usec + (float)$sec);
}
$start = getmicrotime();

// Force UnixTs engine
define('CALENDAR_ENGINE','UnixTs');

if (!@include 'Calendar'.DIRECTORY_SEPARATOR.'Calendar.php') {
    define('CALENDAR_ROOT', '../../');
}
require_once CALENDAR_ROOT.'Month/Weeks.php';
require_once CALENDAR_ROOT.'Day.php';

// Initialize GET variables if not set
if (!isset($_GET['y'])) $_GET['y'] = date('Y');
if (!isset($_GET['m'])) $_GET['m'] = date('m');
if (!isset($_GET['d'])) $_GET['d'] = date('d');

// Build a month object
$month = new Calendar_Month_Weeks($_GET['y'], $_GET['m']);

// Create an array of days which are "selected"
// Used for Week::build() below
$selectedDays = array (
    new Calendar_Day($_GET['y'],$_GET['m'], $_GET['d']),
    new Calendar_Day($_GET['y'], 12, 25),
    new Calendar_Day(date('Y'), date('m'), date('d')),
    );

// Instruct month to build Week objects
$month->build();
// Construct strings for next/previous links
$prevStamp = $month->prevMonth(true); // Passing true returns timestamp, without returns numeric value of month
$prev = $_SERVER['PHP_SELF'].'?y='.date('Y',$prevStamp).'&m='.date('n',$prevStamp).'&d='.date('j',$prevStamp);
$nextStamp = $month->nextMonth(true);
$next = $_SERVER['PHP_SELF'].'?y='.date('Y',$nextStamp).'&m='.date('n',$nextStamp).'&d='.date('j',$nextStamp);
?>
<!doctype html public "-//W3C//DTD HTML 4.0 Transitional//EN">
<html>
<head>
<title> Calendar </title>
<style text="text/css">
table {
    background-color: silver;
}
caption {
    font-family: verdana;
    font-size: 12px;
    background-color: while;
}
.prevMonth {
    font-size: 10px;
    text-align: left;
}
.nextMonth {
    font-size: 10px;
    text-align: right;
}
th {
    font-family: verdana;
    font-size: 11px;
    color: navy;
    text-align: right;
}
td {
    font-family: verdana;
    font-size: 11px;
    text-align: right;
}
.selected {
    background-color: yellow;
}
.empty {
    color: white;
}
</style>
</head>

<body>
<h2>Build with Calendar_Month_Weeks::build() then Calendar_Week::build()</h2>
<table class="calendar">
<caption>
<?php echo date('F Y', $month->getTimeStamp()); ?>
</caption>
<tr>
<th>M</th>
<th>T</th>
<th>W</th>
<th>T</th>
<th>F</th>
<th>S</th>
<th>S</th>
</tr>
<?php
while ($week = $month->fetch()) {
    echo "<tr>\n";
    // Build the days in the week, passing the selected days
    $week->build($selectedDays);
    while ($day = $week->fetch()) {
        // Build a link string for each day
        $dayStamp = $day->thisDay(true);
        $link = $_SERVER['PHP_SELF'].'?y='.date('Y', $dayStamp).
            '&m='.date('m',$dayStamp).'&d='.date('d', $dayStamp);
        // Check to see if day is selected
        if ($day->isSelected()) {
            echo '<td class="selected">'.$day->thisDay().'</td>'."\n";
        // Check to see if day is empty
        } else if ($day->isEmpty()) {
            echo '<td class="empty">'.$day->thisDay().'</td>'."\n";
        } else {
            echo '<td><a href="'.$link.'">'.$day->thisDay().'</a></td>'."\n";
        }
    }
    echo '</tr>'."\n";
}
?>
<tr>
<td>
<a href="<?php echo $prev; ?>" class="prevMonth"><< </a>
</td>
<td colspan="5">&nbsp;</td>
<td>
<a href="<?php echo $next; ?>" class="nextMonth"> >></a>
</td>
</tr>
</table>
<?php
echo '<p><b>Took: '.(getmicrotime()-$start).' seconds</b></p>';
?>
</body>
</html>